from __future__ import annotations  # c.f. PEP 563, PEP 649

import os
import ssl
from typing import TYPE_CHECKING
from urllib.request import urlretrieve

from pytest import fixture

if TYPE_CHECKING:
    from pytest import Config


def pytest_configure(config: Config) -> None:
    """Configure pytest options."""
    warnings_lines = r"""
    error::
    """
    for warning_line in warnings_lines.split("\n"):
        warning_line = warning_line.strip()
        if warning_line and not warning_line.startswith("#"):
            config.addinivalue_line("filterwarnings", warning_line)


"""
The following fetch_file and configuration test module was taken from phys2bids.
Credit to the original author(s) and to the phys2bids community.
"""


def fetch_file(osf_id, path, filename):
    """
    Fetch file located on OSF and downloads to `path`/`filename`.

    Parameters
    ----------
    osf_id : str
        Unique OSF ID for file to be downloaded. Will be inserted into relevant
        location in URL: https://osf.io/{osf_id}/download
    path : str
        Path to which `filename` should be downloaded. Ideally a temporary
        directory
    filename : str
        Name of file to be downloaded (does not necessarily have to match name
        of file on OSF)

    Returns
    -------
    full_path : str
        Full path to downloaded `filename`
    """
    # This restores the same behavior as before.
    # this three lines make tests downloads work in windows
    if os.name == "nt":
        orig_sslsocket_init = ssl.SSLSocket.__init__
        ssl.SSLSocket.__init__ = (
            lambda *args, cert_reqs=ssl.CERT_NONE, **kwargs: orig_sslsocket_init(
                *args, cert_reqs=ssl.CERT_NONE, **kwargs
            )
        )
        ssl._create_default_https_context = ssl._create_unverified_context
    url = "https://osf.io/{}/download".format(osf_id)
    full_path = os.path.join(path, filename)
    if not os.path.isfile(full_path):
        urlretrieve(url, full_path)
    return full_path


@fixture(scope="session")
def testdir(tmp_path_factory):
    """Test path that will be used to download all files."""
    return tmp_path_factory.getbasetemp()


@fixture(scope="function")
def atlas(testdir):
    return fetch_file("h6nj7", testdir, "atlas.nii.gz")


@fixture(scope="function")
def atlastime(testdir):
    return fetch_file("ts6a8", testdir, "ats.nii.gz")


@fixture(scope="function")
def mean_fc(testdir):
    return fetch_file("jrg8d", testdir, "mean_fc_matlab.tsv")


@fixture(scope="function")
def sdi(testdir):
    return fetch_file("rs4dn", testdir, "SDI_matlab.tsv")


@fixture(scope="function")
def sc_mtx(testdir):
    return fetch_file("vwh75", testdir, "sc.mat")


@fixture(scope="function")
def timeseries(testdir):
    return fetch_file("ay8df", testdir, "func.mat")



from . import (
    blocks,
    cli,
    due,
    io,
    objects,
    operations,
    references,
    utils,
    viz,
    workflow,
)
from ._version import get_versions

__version__ = get_versions()["version"]
del get_versions



#!/usr/bin/env python3
"""Tests for blocks."""

import shutil
import sys
from os import makedirs, remove
from os.path import isfile, join

import nibabel
import numpy as np
import pytest
from pymatreader import read_mat

from nigsp import blocks
from nigsp.io import load_nifti_get_mask
from nigsp.objects import SCGraph


# ### Unit tests
def test_nifti_to_timeseries(atlastime, atlas):
    img_in = nibabel.load(atlas)
    atlas_in = img_in.get_fdata()
    ts_out, atlas_out, img_out = blocks.nifti_to_timeseries(atlastime, atlas)
    ts_in = np.unique(atlas_in)[1:, np.newaxis]
    ts_in = np.repeat(ts_in, 2, axis=-1)

    assert (ts_in == ts_out).all()
    assert (atlas_in == atlas_out).all()
    assert (img_in.header["dim"] == img_out.header["dim"]).all()

    remove(atlastime)
    remove(atlas)


@pytest.mark.parametrize("ext", [(".nii.gz"), (".csv")])
def test_export_metrics_txt(ext, sc_mtx, atlas, sdi, testdir):
    testdir = join(testdir, "testdir")
    makedirs(testdir, exist_ok=True)
    atlas_in, mask, img = load_nifti_get_mask(atlas, ndim=3)
    mtx = read_mat(sc_mtx)
    mtx = mtx["SC_avg56"]
    sdi_in = np.genfromtxt(sdi)
    ts = np.unique(atlas_in)[1:]
    ts = np.repeat(ts[..., np.newaxis], 2, axis=-1)
    gsdi = dict.fromkeys(["hilo", "himid"], sdi_in)
    scgraph = SCGraph(mtx, ts, atlas=atlas_in, img=img, sdi=sdi_in)
    blocks.export_metric(scgraph, ext, join(testdir, "molly_"))
    scgraph = SCGraph(mtx, ts, atlas=atlas_in, img=img, gsdi=gsdi)
    blocks.export_metric(scgraph, ext, join(testdir, "cesar_"))

    assert isfile(join(testdir, f"molly_sdi{ext}"))
    assert isfile(join(testdir, f"cesar_gsdi_hilo{ext}"))
    assert isfile(join(testdir, f"cesar_gsdi_himid{ext}"))

    shutil.rmtree(testdir)
    remove(sc_mtx)
    remove(atlas)
    remove(sdi)


def test_export_metrics_nifti(sc_mtx, atlas, sdi, testdir):
    testdir = join(testdir, "testdir")
    makedirs(testdir, exist_ok=True)
    atlas_in, mask, img = load_nifti_get_mask(atlas, ndim=3)
    mtx = read_mat(sc_mtx)
    mtx = mtx["SC_avg56"]
    sdi_in = np.genfromtxt(sdi)
    ts = np.unique(atlas_in)[1:]
    ts = np.repeat(ts[..., np.newaxis], 2, axis=-1)
    scgraph = SCGraph(mtx, ts, atlas=atlas_in, sdi=sdi_in)
    blocks.export_metric(scgraph, ".nii.gz", join(testdir, "molly_"))

    assert isfile(join(testdir, "molly_sdi.tsv.gz"))

    sys.modules["nibabel"] = None
    scgraph = SCGraph(mtx, ts, atlas=atlas_in, img=img, sdi=sdi_in)
    blocks.export_metric(scgraph, ".nii.gz", join(testdir, "molly_"))
    sys.modules["nibabel"] = nibabel

    assert isfile(join(testdir, "molly_sdi.tsv.gz"))
    shutil.rmtree(testdir)
    remove(sc_mtx)
    remove(atlas)
    remove(sdi)


def test_plot_metrics(atlas, sc_mtx, sdi, testdir):
    pytest.importorskip("nilearn")
    from nilearn.plotting import find_parcellation_cut_coords

    testdir = join(testdir, "testdir")
    makedirs(testdir, exist_ok=True)
    atlas_in, _, img = load_nifti_get_mask(atlas, ndim=3)
    mtx = read_mat(sc_mtx)
    mtx = mtx["SC_avg56"]
    sdi_in = np.genfromtxt(sdi)
    ts = np.unique(atlas_in)[1:]
    ts = np.repeat(ts[..., np.newaxis], 2, axis=-1)
    gsdi = dict.fromkeys(["hilo", "himid"], sdi_in)

    scgraph = SCGraph(mtx, ts, sdi=sdi_in, img=img)
    blocks.plot_metric(scgraph, join(testdir, "maite_"), atlas=img)
    assert isfile(join(testdir, join(testdir, "maite_sdi.png")))

    atlas_in = find_parcellation_cut_coords(img)
    scgraph = SCGraph(mtx, ts, sdi=sdi_in, img=img)
    blocks.plot_metric(scgraph, join(testdir, "maite_"), atlas=atlas_in)
    assert isfile(join(testdir, join(testdir, "maite_sdi.png")))

    scgraph = SCGraph(mtx, ts, gsdi=gsdi, img=img)
    blocks.plot_metric(scgraph, join(testdir, "dimitri_"), atlas=img)
    assert isfile(join(testdir, join(testdir, "dimitri_gsdi_hilo.png")))

    shutil.rmtree(testdir)
    remove(sc_mtx)
    remove(atlas)
    remove(sdi)



#!/usr/bin/env python3
"""Tests for viz."""

import sys
from os import remove
from os.path import isfile

import matplotlib
import pytest
from numpy import genfromtxt
from numpy.random import rand

from nigsp import viz


@pytest.mark.parametrize(
    "mtx", [(rand(3, 4)), (rand(4, 4)), (rand(4, 4, 3)), (rand(4, 4, 3, 1))]
)
# ### Unit tests
def test_plot_connectivity(mtx):
    pytest.importorskip("nilearn")

    viz.plot_connectivity(mtx, "annie.png", closeplot=True)

    assert isfile("annie.png")
    matplotlib.pyplot.close()
    remove("annie.png")


@pytest.mark.parametrize(
    "timeseries", [(rand(3, 50)), (rand(3, 50, 3)), (rand(3, 50, 4, 1))]
)
def test_plot_greyplot(timeseries):
    viz.plot_greyplot(timeseries, "troy.png", closeplot=True)

    assert isfile("troy.png")
    matplotlib.pyplot.close()
    remove("troy.png")


def test_plot_nodes(sdi, atlas):
    pytest.importorskip("nilearn")

    ns = genfromtxt(sdi)
    viz.plot_nodes(ns, atlas, "abed.png", closeplot=True)

    assert isfile("abed.png")
    matplotlib.pyplot.close()
    remove("abed.png")
    remove(sdi)
    remove(atlas)


def test_plot_edges(atlas):
    pytest.importorskip("nilearn")

    mtx = rand(360, 360)
    mtx = mtx - mtx.min() + 0.3
    with pytest.warns(UserWarning, match="'adjacency_matrix' is not symmetric"):
        viz.plot_edges(mtx, atlas, "britta.png", thr="95%", closeplot=True)

    assert isfile("britta.png")
    matplotlib.pyplot.close()
    remove("britta.png")
    remove(atlas)


# ### Break tests
def test_break_plot_connectivity():
    pytest.importorskip("nilearn")

    import nilearn.plotting

    sys.modules["matplotlib"] = None
    with pytest.raises(ImportError) as errorinfo:
        viz.plot_connectivity(rand(3, 3), "craig.png")
    assert "are required" in str(errorinfo.value)
    sys.modules["matplotlib"] = matplotlib

    sys.modules["nilearn.plotting"] = None
    with pytest.raises(ImportError) as errorinfo:
        viz.plot_connectivity(rand(3, 3), "craig.png")
    assert "are required" in str(errorinfo.value)
    sys.modules["nilearn.plotting"] = nilearn.plotting

    with pytest.raises(ValueError) as errorinfo:
        viz.plot_connectivity(rand(3, 3, 3, 4), "craig.png")
    assert "plot connectivity" in str(errorinfo.value)

    matplotlib.pyplot.close()


def test_break_plot_greyplot():
    sys.modules["matplotlib"] = None
    with pytest.raises(ImportError) as errorinfo:
        viz.plot_greyplot(rand(3, 3), "dan.png")
    assert "is required" in str(errorinfo.value)
    sys.modules["matplotlib"] = matplotlib

    with pytest.raises(ValueError) as errorinfo:
        viz.plot_greyplot(rand(3, 3, 3, 4), "dan.png")
    assert "plot greyplots" in str(errorinfo.value)

    matplotlib.pyplot.close()


def test_break_plot_nodes(atlas):
    pytest.importorskip("nilearn")

    import nilearn.plotting

    sys.modules["matplotlib"] = None
    with pytest.raises(ImportError) as errorinfo:
        viz.plot_nodes(rand(3), rand(3, 3))
    assert "are required" in str(errorinfo.value)
    sys.modules["matplotlib"] = matplotlib

    sys.modules["nilearn.plotting"] = None
    with pytest.raises(ImportError) as errorinfo:
        viz.plot_nodes(rand(3), rand(3, 3))
    assert "are required" in str(errorinfo.value)
    sys.modules["nilearn.plotting"] = nilearn.plotting

    with pytest.raises(ValueError) as errorinfo:
        viz.plot_nodes(rand(3, 3, 4), rand(3, 3))
    assert "plot node values" in str(errorinfo.value)

    with pytest.raises(NotImplementedError) as errorinfo:
        viz.plot_nodes(rand(3), rand(3, 3, 2))
    assert "atlases in nifti" in str(errorinfo.value)
    with pytest.raises(NotImplementedError) as errorinfo:
        viz.plot_nodes(rand(3), rand(3, 4))
    assert "atlases in nifti" in str(errorinfo.value)

    with pytest.raises(ValueError) as errorinfo:
        viz.plot_nodes(rand(3), rand(4, 3))
    assert "different length" in str(errorinfo.value)

    matplotlib.pyplot.close()
    remove(atlas)


def test_break_plot_edges(atlas):
    pytest.importorskip("nilearn")

    import nilearn.plotting

    sys.modules["matplotlib"] = None
    with pytest.raises(ImportError) as errorinfo:
        viz.plot_edges(rand(3), rand(3, 3))
    assert "are required" in str(errorinfo.value)
    sys.modules["matplotlib"] = matplotlib

    sys.modules["nilearn.plotting"] = None
    with pytest.raises(ImportError) as errorinfo:
        viz.plot_edges(rand(3), rand(3, 3))
    assert "are required" in str(errorinfo.value)
    sys.modules["nilearn.plotting"] = nilearn.plotting

    with pytest.raises(ValueError) as errorinfo:
        viz.plot_edges(rand(3, 3, 4, 5), rand(3, 3))
    assert "plot node values" in str(errorinfo.value)

    with pytest.raises(NotImplementedError) as errorinfo:
        viz.plot_edges(rand(3), rand(3, 3, 2))
    assert "atlases in nifti" in str(errorinfo.value)
    with pytest.raises(NotImplementedError) as errorinfo:
        viz.plot_edges(rand(3), rand(3, 4))
    assert "atlases in nifti" in str(errorinfo.value)

    with pytest.raises(ValueError) as errorinfo:
        viz.plot_edges(rand(3), rand(4, 3))
    assert "different length" in str(errorinfo.value)

    matplotlib.pyplot.close()
    remove(atlas)


# ..and a movie!!!
